import { createContext, useContext, useState, useEffect } from 'react'
import environment from '@/utils/environment'

export const RouteContext = createContext()

export const useRoute = () => {
  return useContext(RouteContext)
}

export const RouteProvider = ({ children }) => {
  const [breadcrumbsList, setBreadcrumbsList] = useState([])

  useEffect(() => {
    const storedBreadcrumbs = JSON.parse(sessionStorage.getItem('breadcrumbsList')) || []
    if (storedBreadcrumbs.length !== 0) {
      setBreadcrumbsList(storedBreadcrumbs)
    }
  }, [])

  useEffect(() => {
    sessionStorage.setItem('breadcrumbsList', JSON.stringify(breadcrumbsList))
  }, [breadcrumbsList])

  const addBreadcrumb = (title, location) => {
    const timestamp = '/' + environment.getTime()
    if (location.endsWith(timestamp)) {
      location = location.replace(timestamp, '')
    } else if (location.endsWith('[...index]')) {
      location = location.replace('/[...index]', '')
    }

    setBreadcrumbsList(prev => {
      const lastBreadcrumb = prev[prev.length - 1]
      if (lastBreadcrumb && lastBreadcrumb.location === location && lastBreadcrumb.title === title) {
        return prev
      }
      return [...prev, { title, location }]
    })
  }

  const breadcrumbsListArr = () => {
    return breadcrumbsList.slice(0, breadcrumbsList.length - 1)
  }

  const truncateBreadcrumbs = (index) => {
    setBreadcrumbsList(prev => prev.slice(0, index))
  }

  useEffect(() => {
    const handlePopState = () => {
      setBreadcrumbsList(prev => {
        const newArr = [...prev]
        newArr.pop()
        return newArr
      })
    }

    window.addEventListener('popstate', handlePopState)
    return () => {
      window.removeEventListener('popstate', handlePopState)
    }
  }, [])

  const value = {
    breadcrumbsListArr,
    addBreadcrumb,
    truncateBreadcrumbs,
    breadcrumbsList
  }

  return (
    <RouteContext.Provider value={value}>
      {children}
    </RouteContext.Provider>
  )
}
